(*************************************************************************)
(*                                                                       *)
(*                     Mouse Trajectory Unit                             *)
(*                                                                       *)
(*                                                                       *)
(* Author: Arash Partow                                                  *)
(* Copyright notice:                                                     *)
(*                                                                       *)
(* Free use of the Mouse Trajectory Unit is permitted under the          *)
(* guidelines and in accordance with the most current version  of the    *)
(* Common Public License.                                                *)
(* http://www.opensource.org/licenses/cpl.php                            *)
(*                                                                       *)
(*************************************************************************)

unit Mouse_Bk;

interface

Uses Math,Graphics,Types;

Type PointXY = record
                x:Double;
                y:Double;
               End;


Type Projectile = Object

      Point: PointXY;
      Constructor Create();  overload;
      Constructor Create(x,y:Double);  overload;
      Constructor Create(cpoint:PointXY); overload;
      Destructor  Destroy();
      Procedure   CalcPath(Bearing,Dist:Double);
      Function    xVal:Double;
      Function    yVal:Double;
      Function    Quadrant(Ang:Double):Integer; overload;
      Function    Quadrant(x,y:Double):Integer; overload;
      Function    xMultiplier(Ang:Double):Integer;
      Function    yMultiplier(Ang:Double):Integer;
      Function    DeltaAngle(Ang:Double):Integer;
      Function    DeltaAngleQ(Quad:Integer):Integer;
     End;



Type TMouse = Object

      Proj: Projectile;
      ox:Double;
      oy:Double;
      DestX:Double;
      DestY:Double;
      T_X:Integer;
      T_Y:Integer;
      Constructor Create();  overload;
      Constructor Create(x,y:Double);  overload;
      Constructor Create(x,y:Double; tx,ty:Integer);  overload;
      Procedure MouseChase(x,y,Dist:Double);
      Procedure DrawMouse(Canvas: TCanvas);
      Function xVal:Double;
      Function yVal:Double;
      Function GetBearing(x,y:Double):Double;
      Function Quadrant(x,y:Double):Integer;
     End;


Const MAX_MICE   = 600;

Const MAX_DELAY  = 10000000;
Const MAX_X      = 610;
Const MAX_Y      = 610;
Const T_X        = 305;
Const T_Y        = 305;


Var Mouse: Array[0..MAX_MICE] Of TMouse;


Procedure InitialiseMice;
Procedure DoMouseChase;
Procedure DrawMice(Canvas:TCanvas);
Procedure DrawShapeBoarder(Canvas:TCanvas);
Procedure SetupGraphicScreen(Canvas:TCanvas);
Procedure InterceptComplete(Canvas:TCanvas);
Function  Distance(x1,y1,x2,y2:Double):Double;

Var  MICE:Integer;

implementation


Constructor Projectile.Create();
Begin
End;


Constructor Projectile.Create(x,y:Double);
Begin
 point.x := x;
 point.y := y;
End;


Constructor Projectile.Create(cpoint:PointXY);
Begin
 point:=cpoint;
End;

Destructor Projectile.Destroy;
Begin
End;

Function Projectile.xVal:Double;
Begin
 xVal:= point.x;
End;


Function Projectile.yVal:Double;
Begin
 yVal:= point.y;
End;


Procedure Projectile.CalcPath(Bearing,Dist:Double);
Var
  xMult  : Integer;
  yMult  : Integer;
  Quad   : Integer;
  RAng   : Double;
  BFrac  : Double;
Begin
 xMult  := xMultiplier(Bearing);
 yMult  := yMultiplier(Bearing);
 Quad   := Quadrant(Bearing);
 BFrac  := Frac(Bearing);
 RAng   := (Trunc(Bearing) Mod 90)+BFrac;
 If Quad Mod 2 = 0 Then
  Begin
   point.x := point.x + cos(RAng*PI/180)*dist*xMult;
   point.y := point.y + sin(RAng*PI/180)*dist*yMult;
  End
   Else
    Begin
     point.x := point.x + sin(RAng*PI/180)*dist*xMult;
     point.y := point.y + cos(RAng*PI/180)*dist*yMult;
    End;
End;


Function Projectile.Quadrant(Ang:Double):Integer;
Begin
 If(Ang >=   0) And (Ang <  90) Then Result := 1
  Else
   If(Ang >= 270) And (Ang < 360) Then Result := 2
    Else
     If(Ang >= 180) And (Ang < 270) Then Result := 3
      Else
       If(Ang >=  90) And (Ang < 180) Then Result := 4
        Else
         Result := 0;
End;


Function Projectile.Quadrant(x,y:Double):Integer;
Begin
 If (0 <= x) And (0 <= y) Then Result := 1
  Else
   If (x <  0) And (0 <= y) Then Result := 2
    Else
     If (x <  0) And (y <  0) Then Result := 3
      Else
       If (0 <= x) And (y <  0) Then Result := 4
        Else
         Quadrant := 0;
End;

Function Projectile.xMultiplier(Ang:Double):Integer;
 Begin
  Case Quadrant(Ang) Of
   1: Result:=  1;
   2: Result:= -1;
   3: Result:= -1;
   4: Result:=  1;
   Else
    Result:=0;
  End;
 End;


Function Projectile.yMultiplier(Ang:Double):Integer;
Begin
 Case Quadrant(Ang) Of
  1: Result :=  1;
  2: Result :=  1;
  3: Result := -1;
  4: Result := -1;
   Else
    Result := 0;
 End;
End;



Function Projectile.DeltaAngle(Ang:Double):Integer;
Begin
 Case  Quadrant(Ang) Of
   1: Result := 0;
   2: Result := 270;
   3: Result := 180;
   4: Result := 90;
    Else
     Result:=0;
 End;
End;


Function Projectile.DeltaAngleQ(Quad:Integer):Integer;
Begin
 Case Quad Of
   1: Result := 0;
   2: Result := 270;
   3: Result := 180;
   4: Result := 90;
   Else
    Result:=0;
 End;
End;



Constructor TMouse.Create();
Begin
 Proj.Create();
 ox:=0;
 oy:=0;
 T_X:=0;
 T_Y:=0;
 DestX:=0;
 DestY:=0;
End;


Constructor TMouse.Create(x,y:Double);
Begin
 proj.Create(x,y);
 ox := x;
 oy := y;
End;


Constructor TMouse.Create(x,y:Double; tx,ty:Integer);
Begin
 Create(x,y);
 T_X:=tx;
 T_Y:=ty;
 DestX:=0;
 DestY:=0;
End;


Procedure TMouse.MouseChase(x,y,Dist:Double);
Var Bearing:Double;
Begin
 Bearing := GetBearing(x,y);
 DestX := x;
 DestY := y;
 proj.calcPath(Bearing,dist);
End;

Procedure TMouse.DrawMouse(Canvas: TCanvas);
Var x1,y1,x2,y2:Integer;
Begin

 Canvas.Brush.Color := clBlack;
 Canvas.Pen.Color := clBlack;
 x1:= Round(ox-4+T_X);
 y1:= Round(-oy-4+T_Y);
 x2:= Round(ox+4+T_X);
 y2:= Round(-oy+4+T_Y);
 Canvas.Ellipse(x1,y1,x2,y2);

 Canvas.Brush.Color := clLime;
 Canvas.Pen.Color := clBlack;
 x1:= Round(Proj.xVal-4+T_X);
 y1:= Round(-Proj.yVal-4+T_Y);
 x2:= Round(Proj.xVal+4+T_X);
 y2:= Round(-Proj.yVal+4+T_Y);
 Canvas.Ellipse(x1,y1,x2,y2);
 ox := Proj.xVal();
 oy := Proj.yVal();
End;


Function TMouse.xVal:Double;
Begin
 xVal:= Proj.xVal;
End;


Function TMouse.yVal:Double;
Begin
 yVal:= Proj.yVal;
End;


Function TMouse.GetBearing(x,y:Double):Double;
Var
 nx  : Double;
 ny  : Double;
 Quad: Integer;
Begin
 nx   := x - Proj.xVal;
 ny   := y - Proj.yVal;
 Quad := Quadrant(nx,ny);
 If (Quad = 1 ) Then Result:= abs(ArcTan2(nx,ny))*180/PI
  Else
   If (Quad = 2 ) Then Result:= abs(ArcTan2(ny,-nx))*180/PI+270
    Else
     If (Quad = 3 ) Then Result:= abs(ArcTan2(-nx,-ny))*180/PI+180
      Else
       If (Quad = 4 ) Then Result:= abs(ArcTan2(-ny,nx))*180/PI+90
        Else
         Result := 0;
End;


Function TMouse.Quadrant(x,y:Double):Integer;
Begin
 If (0 <= x) And (0 <= y) Then Result:= 1
  Else
   If (x <  0) And (0 <= y) Then Result:= 2
    Else
     If (x <  0) And (y <  0) Then Result:= 3
      Else
        If (0 <= x) And (y <  0) Then Result:= 4
         Else
          Result := 0;
End;


Procedure InitialiseMice;
Var
 I   : Integer;
 Ang : Double;
 Rang: Double;
 Proj: Projectile;
Begin
 Ang   := 360 / MICE;
 Rang  := 0;
 For I:= 0 To MICE-1 Do
  Begin
   Proj.Create(0,0);
   Proj.CalcPath(Round(360-rang) Mod 360,Round((MAX_X-5*(MAX_X*0.105))));
   Mouse[i].Create(Proj.xVal,Proj.yVal,T_X,T_Y);
   RAng := RAng + Ang;
  End;
End;


Procedure DoMouseChase;
Var I    : Integer;
    Dist : Double;
Begin
 For I:= 0 To MICE-1 Do
  Begin
   Dist := Distance(Mouse[i].Proj.xVal,Mouse[i].Proj.yVal, Mouse[(i+1) Mod MICE].ox,Mouse[(i+1) Mod MICE].oy)*0.02;
   Mouse[i].MouseChase(Mouse[(i+1) Mod MICE].ox,Mouse[(i+1) Mod MICE].oy,Dist);
  End;
End;


Procedure DrawMice(Canvas:TCanvas);
Var I:Integer;
Begin
 For I:= 0 To MICE-1 Do
  Begin
   Mouse[I].DrawMouse(Canvas);
  End;
End;


Procedure DrawShapeBoarder(Canvas:TCanvas);
Var I: Integer;
Begin
 Canvas.Brush.Color := clBlack;
 Canvas.Pen.Color   := clBlack;
 For I:= 0 To MICE-1 Do
  Begin
   Canvas.MoveTo(Round(Mouse[i].proj.xVal+T_X),Round(-1*Mouse[i].proj.yVal+T_Y));
   Canvas.LineTo(Round(Mouse[(i+1) Mod MICE].proj.xVal+T_X),Round(-1*Mouse[(i+1) Mod MICE].proj.yVal)+T_Y);
  End;
End;

Function Distance(x1,y1,x2,y2:Double):Double;
Begin
 Result:=sqrt(sqr(x1-x2)+sqr(y1-y2));
End;

Procedure SetupGraphicScreen(Canvas:TCanvas);
Begin
 Canvas.Brush.Color := clWhite;
 Canvas.FillRect(Canvas.ClipRect);
End;

Procedure InterceptComplete(Canvas:TCanvas);
Var Str:String;
Begin
 Canvas.Brush.Color := clWhite;
 Canvas.Font.Color:=clRed;
 Canvas.Font.Size:=18;
 Str:='INTERCEPT COMPLETE';
 With Canvas.ClipRect Do
  Begin
   Canvas.Brush.style := bsClear;
   Canvas.TextOut(((Right-Left)-Canvas.TextWidth(Str)) Div 2, ((Bottom-Top)-Canvas.TextHeight(Str)) Div 2,Str);
  End;
End;


end.
